/**
 * Element einer Menge oder Liste.
 * 
 * @author Owyn
 * @version 2007-05-28
 */
class Element {
	int data; // Inhalt des Elements
	Element next; // Naechstes Element
	
    /**
     * Erzeugt ein neues Element.
     * 
     * @param data Inhalt des Elements
     * @param next Verweis auf das Nachfolgeelement
     */
	Element(int data, Element next) {
	    this.data = data;
	    this.next = next;
	}

    /**
     * Startet den Test.
     * 
     * @param args wird ignoriert
     */
	Element(int data) {
	    this(data, null);
	}

}

/**
 * Menge von Ganzzahlen.
 * 
 * @author Owyn
 * @version 2007-05-28
 */
public class Menge {
	private Element firstElement; // Erstes Element der Menge

    /**
     * Zugriff auf das erste Element der Menge von auen.
     * 
     * @return Verweis auf das erste Element
     */
	public Element firstElement() { return firstElement; }

    /**
     * Gibt die Anzahl der Elemente der Menge zurck.
     * 
     * @return Anzahl der Elemente
     */
	public int groesse() {
		Element current = firstElement;
		int count = 0;
		while (current != null) { // Menge durchlaufen
			count++; // Zaehlen
			current = current.next; // Naechstes Element
		}
		return count; // Anzahl zurueckgeben
	}

    /**
     * Testet, ob eine Ganzzahl schon in der Menge enthalten ist.
     * 
     * @param data Ganzzahl
     * @return true, wenn data in der Menge enthalten ist, sonst false.
     */
	public boolean istEnthalten(int data) {
		Element current = firstElement;
		while (current != null) { // Menge durchlaufen
			if (current.data == data) return true; // Gefunden
			current = current.next; // Naechstes Element
		}
		return false; // Nicht gefunden
	}

    /**
     * Fgt eine Ganzzahl in die Menge ein, sofern sie noch nicht vorhanden ist.
     * 
     * @param data Ganzzahl
     * @return true, wenn das Element eingefgt wurde, sonst false.
     */
	public boolean fuegeEin(int data) {
		if (istEnthalten(data)) return false; // Element schon vorhanden?

		// Wenn nein: neues Element erzeugen und vor dem ersten Element einfgen
		Element newElement = new Element(data, firstElement);
		firstElement = newElement; // Erstes Element der Menge aktualisieren
		// auch mglich: firstElement = new Element(data, firstElement);
		
		return true; // Element wurde eingefuegt
	}

    /**
     * Entfernt eine Ganzzahl in die Menge ein, sofern sie vorhanden ist.
     * 
     * @param data Ganzzahl
     * @return true, wenn das Element entfernt wurde, sonst false.
     */
	public boolean entferne(int data) {
		if (firstElement == null) return false; // Leere Menge
		if (firstElement.data == data) { // Als erstes Element gefunden
			firstElement = firstElement.next; // Entfernen
			return true;
		}
		Element current = firstElement;
		while (current.next != null) { // Menge durchlaufen
			if (current.next.data == data) { // Gefunden
				current.next = current.next.next; // Entfernen
				return true;
			}
			current = current.next; // Naechstes Element
		}
		return false; // Nicht gefunden
	}

    /**
     * Gibt eine Text-Darstellung der Menge zurck.
     * 
     * @return Darstellung nach dem Muster "1 -> 2 -> (X)"
     */
	public String toString() {
		StringBuffer str = new StringBuffer(); // Leerer String(buffer)
		Element current = firstElement;
		while (current != null) { // Menge durchlaufen
			str.append(current.data);
			str.append(" -> ");
			current = current.next; // Naechstes Element
		}
		str.append("(X)");
		return str.toString();
	}

    /**
     * Vereinigt die aktuelle Menge mit der Menge m.
     * 
     * @param m Menge (wird verndert!)
     * @return nichts, aber die Menge m enthlt die Vereinigungsmenge.
     */
	public void union(Menge m) {
		Element current = firstElement;
		while (current != null) { // Menge durchlaufen
			m.fuegeEin(current.data); // Element in m einfuegen
			current = current.next; // Naechstes Element
		}
	}


    /**
     * Bildet die Schnittmenge der aktuellen Menge mit der Menge m.
     * 
     * @param m Menge (wird verndert!)
     * @return nichts, aber die Menge m enthlt die Schnittmenge.
     */
	public void intersect(Menge m) {
		Element current = firstElement; // Erstes Element der aktuellen Menge
		while (current != null) { // Aktuelle Menge durchlaufen
			// Element nur in m lassen, wenn es auch in der aktuellen Menge ist
			if (!m.istEnthalten(current.data)) m.entferne(current.data);
			current = current.next; // Naechstes Element
		}
		current = m.firstElement(); // Erstes Element der Menge m
		while (current != null) { // Menge m durchlaufen
			// Element nur in m lassen, wenn es auch in der aktuellen Menge ist
			if (!istEnthalten(current.data)) m.entferne(current.data);
			current = current.next; // Naechstes Element
		}
	}

}
