/**
 * Element einer Menge oder Liste.
 * 
 * @author Owyn
 * @version 2007-05-28
 */
public class Tree {

	private String data;
	private Tree left;
	private Tree right;

    /**
     * Erzeugt einen neuen Baumknoten.
     * 
     * @param data Inhalt des Knotens
     */
	public Tree(String data) {
		this.data = data;
		left = null;
		right = null;
	}

    /**
     * Testet, ob ein linker Teilbaum existiert.
     * 
     * @return true, falls ein linker Teilbaum existiert, sonst false.
     */
	public boolean hasLeft() {
		return (left != null);
	}

    /**
     * Testet, ob ein rechter Teilbaum existiert.
     * 
     * @return true, falls ein rechts Teilbaum existiert, sonst false.
     */
	public boolean hasRight() {
		return (right != null);
	}


    /**
	 * Fgt einen String in den Baum/Teilbaum ein.
	 * Der String wird nach alphabetischer Ordnung einsortiert.
     * 
     * @param data der einzufgende String
     */
	public void insert(String data) {

		int cmp = this.data.compareTo(data); // Vergleichswert

		if (cmp >= 0) {
			// Uebergebener String ist kleiner oder gleich dem eigenen String:
			// Links einsortieren
			if (hasLeft()) { // Gibt schon einen linken Teilbaum
				left.insert(data); // Dort einfuegen
			} else { // Gibt noch keinen linken Teilbaum
				left = new Tree(data); // Links als Blatt einfuegen
			}
		} else if (cmp < 0) {
			// Uebergebener String ist groesser oder gleich dem eigenen String:
			// Rechts einsortieren
			if (hasRight()) { // Gibt schon einen rechten Teilbaum
				right.insert(data); // Dort einfuegen
			} else { // Gibt noch keinen rechten Teilbaum
				right = new Tree(data); // Rechts als Blatt einfuegen
			}
		}
	}

    /**
	 * Gibt eine Text-Version des Baums zurck.
     * 
     * @return die In-Order-Darstellung des Baums
     */
	// Gibt den Baum als String in In-Order-Darstellung zurueck
	public String toString() {
		String str = "";

		if (hasLeft()) str += left.toString(); // Linker Teilbaum
		str += data + " "; // Eigener String
		if (hasRight()) str += right.toString(); // Rechter Teilbaum

		return str.toString();
	}
}
